import axios from 'axios';
import { getTavilyApiKey } from './config';

interface TavilySearchOptions {
  topic?: 'general' | 'news';
  search_depth?: 'basic' | 'advanced';
  chunks_per_source?: number;
  max_results?: number;
  time_range?: 'day' | 'week' | 'month' | 'year' | 'd' | 'w' | 'm' | 'y';
  days?: number;
  include_answer?: boolean | 'basic' | 'advanced';
  include_raw_content?: boolean;
  include_images?: boolean;
  include_image_descriptions?: boolean;
  include_domains?: string[];
  exclude_domains?: string[];
}

interface TavilySearchResult {
  title: string;
  url: string;
  content: string;
  score: number;
  raw_content?: string;
}

interface TavilySearchResponse {
  query: string;
  answer?: string;
  images?: Array<{
    url: string;
    description?: string;
  }>;
  results: TavilySearchResult[];
  response_time: string;
}

export const searchTavily = async (
  query: string,
  opts?: TavilySearchOptions,
) => {
  const tavilyApiKey = getTavilyApiKey();
  
  if (!tavilyApiKey) {
    throw new Error('Tavily API key is not configured');
  }

  const url = 'https://api.tavily.com/search';
  
  const response = await axios.post<TavilySearchResponse>(
    url,
    {
      query,
      ...opts,
    },
    {
      headers: {
        'Content-Type': 'application/json',
        'Authorization': `Bearer ${tavilyApiKey}`,
      },
    }
  );

  const results = response.data.results;
  
  // Convert Tavily results to match the format expected by the rest of the application
  const formattedResults = results.map(result => ({
    title: result.title,
    url: result.url,
    content: result.content,
    img_src: undefined, // Tavily doesn't provide image URLs in the standard response
  }));

  return { 
    results: formattedResults, 
    suggestions: [], // Tavily doesn't provide suggestions, so return empty array
    answer: response.data.answer, // Include the AI-generated answer if available
  };
}; 